@@ -33,6 +33,7 @@
 ;;; RFC???? (UNSELECT ext): done
 ;;; RFC2195 (CRAM-MD5 auth): done
 ;;; RFC2086 (ACL ext): done
+;;; RFC2342 (NAMESPACE ext): done
 ;;; RFC2359 (UIDPLUS ext): done
 
 ;;; Todo:
@@ -118,6 +119,9 @@ FETCH and DATA response.")
 (defvar imap-data-capability nil
   "Current server's capability list")
 
+(defvar imap-data-namespace nil
+  "Current server's namespace.")
+
 (defvar imap-data-folder nil
   "Obarray which contains group information.")
 
@@ -167,6 +171,7 @@ it's own authentication.")
 				 (SEARCH . imap-cb-search)
 				 (STATUS . imap-cb-status)
 				 (ACL . imap-cb-acl)
+				 (NAMESPACE . imap-cb-namespace)
 				 (default . imap-cb-default))
   "Alist of IMAP code to function callbacks.")
 
@@ -179,6 +184,7 @@ it's own authentication.")
                       imap-current-message
                       imap-current-server
                       imap-data-capability
+		      imap-data-namespace
                       imap-data-folder
 		      imap-open-stream
 		      imap-auth-method
@@ -223,10 +229,12 @@ any of these to `imap-open-server'.")
 ;	  imap-ok-p
 	  imap-wait-for-tag
 	  imap-capability-get
+	  imap-namespace-get
 	  imap-authinfo-get
           imap-folder-set
           imap-folder-get
           imap-folder-plist
+	  imap-folder-reset
           imap-dispatch
 	  imap-authenticate
 	  imap-authenticate-login
@@ -241,6 +249,7 @@ any of these to `imap-open-server'.")
           imap-cb-bye
           imap-cb-numbered
           imap-cb-capability
+	  imap-cb-namespace
           imap-cb-list
           imap-cb-flags
           imap-cb-fetch
@@ -557,6 +566,46 @@ in order.  See `imap-send-command-wait'."
 	    imap-message-data nil)
       t)))
 
+(defun imap-folder-lsub (&optional reference buffer)
+  "Return a list of strings of subscribed mailboxes on server in
+BUFFER. REFERENCE is the implementation-specific string that has to be
+passed to LSUB."
+  (with-current-buffer (or buffer (current-buffer))
+    (imap-folder-reset)
+    (when (imap-ok-p (imap-send-command-wait 
+		      (concat "LSUB \"" reference "\" \"*\"")))
+      (imap-folder-map 'identity))))
+
+(defun imap-folder-list (&optional root have-delimiter reference buffer)
+  "List all mailboxes that starts with ROOT in BUFFER. If
+HAVE-DELIMITER is non-nil, a hierarchy delimiter is not added to
+ROOT. REFERENCE is the implementation-specific string that has to be
+passed to LIST."
+  (with-current-buffer (or buffer (current-buffer))
+    (imap-folder-reset)
+    ;; Find hierarchy separator
+    (unless have-delimiter
+      (imap-send-command-wait (concat "LIST \"" reference "\" \"" root "\"")))
+    (when (imap-ok-p 
+	   (imap-send-command-wait 
+	    (concat "LIST \"" reference "\" \"" root
+		    (when (and (not have-delimiter) root)
+		      (imap-folder-get 'delimiter root))
+		    "%\"")))
+      (imap-folder-map 'identity))))
+
+(defun imap-folder-subscribe (mailbox &optional buffer)
+  "Send the SUBSCRIBE command on the mailbox to server in
+BUFFER. Returns non-nil if successful."
+  (with-current-buffer (or buffer (current-buffer))
+    (imap-ok-p (imap-send-command-wait (concat "SUBSCRIBE " mailbox)))))
+
+(defun imap-folder-unsubscribe (mailbox &optional buffer)
+  "Send the SUBSCRIBE command on the mailbox to server in
+BUFFER. Returns non-nil if successful."
+  (with-current-buffer (or buffer (current-buffer))
+    (imap-ok-p (imap-send-command-wait (concat "UNSUBSCRIBE " mailbox)))))
+
 ;;; Variable setters and getters
 
 (defun imap-capability-get (&optional buffer)
@@ -567,6 +616,14 @@ in order.  See `imap-send-command-wait'."
 	(setq imap-data-capability '(IMAP2))))
     imap-data-capability))
 
+(defun imap-namespace-get (&optional buffer)
+  "Return server's namespace."
+  (with-current-buffer (or buffer (current-buffer))
+    (unless imap-data-namespace
+      (when (memq 'NAMESPACE (imap-capability-get))
+	(imap-send-command-wait "NAMESPACE")))
+    imap-data-namespace))
+
 (defun imap-folder-plist (&optional folder buffer)
   "Set PROP to VALUE for FOLDER in BUFFER."
   (with-current-buffer (or buffer (current-buffer))
@@ -588,12 +645,19 @@ in order.  See `imap-send-command-wait'."
                      imap-current-folder) imap-data-folder) prop)))
 
 (defun imap-folder-map (func &optional buffer)
-  "Call (func FOLDER) for each folder in `imap-data-folder'."
+  "Call (func FOLDER) for each folder in `imap-data-folder', returning
+a sequence."
   (with-current-buffer (or buffer (current-buffer))
-    (mapatoms 
-     (lambda (s)
-       (funcall func (symbol-name s)))
-     imap-data-folder)))
+    (let (result)
+      (mapatoms 
+       (lambda (s)
+	 (push (funcall func (symbol-name s)) result))
+       imap-data-folder)
+      result)))
+
+(defun imap-folder-reset (&optional buffer)
+  (with-current-buffer (or buffer (current-buffer)) 
+    (setq imap-data-folder (make-vector imap-convenient-group-prime 0))))
 
 ;;; Internal functions
 
@@ -807,7 +871,8 @@ this returns nil."
   ;; the entirety of the block (you should know the end of it...).
 
   ;; The only place we could have a problem is in the arbitrary TEXT
-  ;; following a status response.  So we quote that first.
+  ;; following a status response.  So we quote that first, and be
+  ;; sure to remove all " and \'s that we can't handle.
 
   (let ((leave-brackets 0))
     (goto-char (point-min))
@@ -822,7 +887,7 @@ this returns nil."
         (narrow-to-region (point) (- (point-max) 2))
         (insert "\"")
         (while (re-search-forward "\\\\\\|\"" nil 00)
-          (replace-match "\\\\\\&"))
+          (replace-match "")) ; xxx
         (insert "\""))
       (goto-char (point-min)))
     (goto-char (- (point-max) 2))
@@ -890,7 +955,7 @@ this returns nil."
   (let ((name (if (symbolp name-symbol-or-string)
                   (symbol-name name-symbol-or-string)
                 name-symbol-or-string)))
-    (imap-folder-set 'flags taglist name)
+    (imap-folder-set 'flags (mapcar 'symbol-name taglist) name)
     (imap-folder-set 'delimiter delim name)
     (when (eq code 'LSUB) 
       (imap-folder-set 'subbed t name))))
@@ -956,6 +1021,9 @@ in `imap-message-data'."
 (defun imap-cb-acl (code group &rest acls)
   (imap-folder-set 'acl acls (symbol-name group)))
 
+(defun imap-cb-namespace (code &rest namespace)
+  (setq imap-data-namespace namespace))
+
 (defun imap-cb-status (code folder statuses)
 ;  (check-valid-plist statuses)
   (mapc (lambda (c) (imap-folder-set (car c) (cdr c) (symbol-name folder)))
