@@ -95,7 +95,7 @@
 
 (nnoo-declare nnimap) ; we derive from no one
 
-(defconst nnimap-version "nnimap 0.34")
+(defconst nnimap-version "nnimap 0.37")
 
 ;; Various server variables.
 
@@ -122,15 +122,15 @@ default is an empty string.
 Example:
  '(\"INBOX\" \"Mail/*\" \"alt.sex.*\" (\"~friend/Mail/\" . \"list/*\"))
 
-Also note that currently groups that start with a '.' cause Gnus
-to choke (a patch is pending acceptance), so instead of something
-like \"*\" which might match \".mailboxlist\" you could use
-\"~/*\" which would match \"~/.mailboxlist\" and not cause problems.
+Also note that currently groups that start with a '.' cause Gnus to
+choke, so instead of using something like \"*\" which might match
+\".mailboxlist\" you could use \"~/*\" which would match
+\"~/.mailboxlist\" and not cause problems.
 
 The two wildcards are * and %.  * means match anything, much like in
 shell globbing in Unix.  * does match hierarchy delimiters (. or /, in
 the usual case.)  % is the same as *, but does not match the hierarchy
-symbol.")
+delimiter.")
 
 (defvoo nnimap-list-method "LIST" ; "LSUB"
   "*Function called on IMAP server to list groups.  One of \"LSUB\" or
@@ -387,7 +387,7 @@ BODY."
 (defun nnimap-authenticate (server &optional buffer)
   (with-current-buffer (or buffer (current-buffer))
     ;; Login
-    (let (list alist user passwd ok)
+    (let (list alist ok)
       (and (fboundp 'gnus-parse-netrc)
 	   (setq list (gnus-parse-netrc nnimap-authinfo-file)
 		 alist (or (and (gnus-netrc-get (gnus-netrc-machine 
@@ -529,7 +529,9 @@ BODY."
         ;; Find the article by number
         (nnimap-send-command-wait (format "UID FETCH %d (%s%s)" article part
 					  (if add-peek ".PEEK" "")))
-        (let ((text (funcall (if (fboundp 'string-as-multibyte)
+        (let ((text (funcall (if (and (fboundp 'string-as-multibyte)
+				      (subrp (symbol-function 
+					      'string-as-multibyte)))
 				 'string-as-multibyte
 			       'identity) (imap-message-get article part))))
           (with-current-buffer (or to-buffer nntp-server-buffer)
@@ -695,9 +697,9 @@ BODY."
 
 ;;; Interface functions, optional backend functions
 
-;;; This gets called instead of `nnimap-request-list' when
-;;; `gnus-read-active-file' is 'some instead of t.
-;;; Returns: GROUP HIGHEST LOWEST [ynmxj=] ???
+;; This gets called instead of `nnimap-request-list' when
+;; `gnus-read-active-file' is 'some instead of t.
+;; Returns: GROUP HIGHEST LOWEST [ynmxj=] ???
 (deffoo nnimap-retrieve-groups (groups &optional server)
   (when (nnimap-possibly-change-server server)
     (with-current-buffer nntp-server-buffer (erase-buffer))
@@ -749,7 +751,7 @@ BODY."
 	   ;; we can't return '(1) since this isn't a "list of ranges",
 	   ;; and we can't return '((1)) since gnus-list-of-unread-articles
 	   ;; is buggy so we return '((1 . 1)).
-           (if (and (atom (car read))
+           (if (and (integerp (car read))
 		    (null (cdr read)))
                (list (cons (car read) (car read)))
              read))))
@@ -769,7 +771,7 @@ BODY."
 	      gnus-article-mark-lists)))
   info)
 
-;;; Respond to articles with mail
+;; Respond to articles with mail
 (deffoo nnimap-request-type (group article)
   'mail)
 
@@ -801,6 +803,10 @@ BODY."
       (mapcar (lambda (rule) 
 		(let ((group (car rule))
 		      (regexp (cadr rule)))
+		  ;; Fold continuation lines.
+		  (goto-char (point-min))
+		  (while (re-search-forward "\\(\r?\n[ \t]+\\)+" nil t)
+		    (replace-match " " t t))
 		  (goto-char (point-min))
 		  (when (re-search-forward regexp nil t)
 		    (setq to-groups (cons group to-groups)))))
@@ -926,19 +932,22 @@ BODY."
 	(list (- ms 1) (+ (expt 2 16) ls))
       (list ms ls))))
 
-;; xxx: day should not be zero-padded or blank-padded.
 (defun nnimap-date-days-ago (daysago)
-  "Return date, in format \"03-Aug-98\", for DAYSAGO days ago."
-  (format-time-string "%d-%b-%y" (nnimap-time-substract
-				  (current-time)
-				  (if (fboundp 'days-to-time)
-				      (days-to-time daysago)
-				    (nnmail-days-to-time daysago)))))
+  "Return date, in format \"3-Aug-98\", for DAYSAGO days ago."
+  (let ((date (format-time-string "%d-%b-%y" 
+				  (nnimap-time-substract
+				   (current-time)
+				   (if (fboundp 'days-to-time)
+				       (days-to-time daysago)
+				     (nnmail-days-to-time daysago))))))
+    (if (eq ?0 (string-to-char date))
+	(substring date 1)
+      date)))
 
 (defun nnimap-request-expire-articles-progress (num fetch data)
   (gnus-message 5 "Expiring; marking article %d for deletion..." num))
 
-;;; Notice that we don't actually delete anything, we just mark them deleted.
+;; Notice that we don't actually delete anything, we just mark them deleted.
 (deffoo nnimap-request-expire-articles (articles group &optional server force)
   (let (oldarts (artseq (gnus-compress-sequence articles)))
     (when (and artseq (nnimap-possibly-change-group group server))
@@ -1167,7 +1176,7 @@ on articles, in GROUP."
 	    ((eq nnimap-expunge-on-close 'never)
 	     (imap-unselect-folder))
 	    ((eq nnimap-expunge-on-close 'ask)
-	     (let ((deleted (imap-search "DELETED")))
+	     (when (imap-search "DELETED")
 	       (if (gnus-y-or-n-p (format "Expunge articles in group `%s'? "
 					  imap-current-folder))
 		   (and (nnimap-ok-p (nnimap-send-command-wait "EXPUNGE"))
@@ -1331,7 +1340,7 @@ nil, change to current server."
 	;; Update marks.
 	
 	(let ((types gnus-article-mark-lists) 
-	      type list old-mark new-mark add del)
+	      type old-mark new-mark add del)
 	  (while (setq type (cdr (pop types)))
 	    ;; cache, score, bookmark are not proper flags.
 	    (unless (memq type '(cache score bookmark))
